"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const request = require("request");
const fs = require("fs");
const promises_1 = require("fs/promises");
const path = require("path");
const tar = require("tar");
const binaries_js_1 = require("./binaries.js");
async function download(url, dest) {
    /* Create an empty file where we can save data */
    const file = fs.createWriteStream(dest);
    /* Using Promises so that we can use the ASYNC AWAIT syntax */
    await new Promise((resolve, reject) => {
        request({
            /* Here you should specify the exact link to the file you are trying to download */
            uri: url,
            gzip: true,
        })
            .pipe(file)
            .on('finish', async () => {
            console.log(`${dest} is finished downloading.`);
            resolve();
        })
            .on('error', (error) => {
            reject(error);
        });
    }).catch((error) => {
        console.log(`Something happened: ${error}`);
    });
}
async function downloadBinaries(tag) {
    fs.rmSync(binaries_js_1.binariesDirectory, { recursive: true, force: true });
    fs.mkdirSync(binaries_js_1.binariesDirectory);
    const holochainRunnerFilenames = {
        win32: 'holochain-runner-x86_64-pc-windows-msvc.tar.gz',
        darwin: 'holochain-runner-x86_64-apple-darwin.tar.gz',
        linux: 'holochain-runner-x86_64-unknown-linux-gnu.tar.gz'
    };
    const lairKeystoreFilenames = {
        win32: 'lair-keystore-x86_64-pc-windows-msvc.exe',
        darwin: 'lair-keystore-x86_64-apple-darwin',
        linux: 'lair-keystore-x86_64-unknown-linux-gnu'
    };
    const lairKeystoreUrl = `https://github.com/Sprillow/holochain-runner/releases/download/${tag}/${lairKeystoreFilenames[process.platform]}`;
    await download(lairKeystoreUrl, binaries_js_1.defaultLairKeystoreBinaryPath);
    const holochainRunnerCompressedUrl = `https://github.com/Sprillow/holochain-runner/releases/download/${tag}/${holochainRunnerFilenames[process.platform]}`;
    const compressedTempFilename = path.join(binaries_js_1.binariesDirectory, holochainRunnerFilenames[process.platform]);
    await download(holochainRunnerCompressedUrl, compressedTempFilename);
    await tar.x({ file: compressedTempFilename, cwd: binaries_js_1.binariesDirectory });
    fs.rmSync(compressedTempFilename);
    // defaultHolochainRunnerBinaryPath
    await promises_1.chmod(binaries_js_1.defaultLairKeystoreBinaryPath, 511);
    await promises_1.chmod(binaries_js_1.defaultHolochainRunnerBinaryPath, 511);
}
;
(async () => {
    try {
        // current holochain-runner release version
        // version-bump
        const holochainRunnerTag = 'v0.0.39';
        await downloadBinaries(holochainRunnerTag);
    }
    catch (e) {
        console.log(e);
    }
})();
//# sourceMappingURL=downloadBinaries.js.map